# -*- coding: utf-8 -*-
from collective.fingerpointing.testing import INTEGRATION_TESTING
from collective.fingerpointing.utils import get_request_information
from plone.app.testing import TEST_USER_NAME

import unittest


class UtilsTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def test_get_request_information(self):
        request = self.layer['request']
        request['AUTHENTICATED_USER'] = TEST_USER_NAME
        request['REMOTE_ADDR'] = localhost = '127.0.0.1'
        self.assertEqual(
            get_request_information(),
            (TEST_USER_NAME, localhost),
        )



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import fingerpointing_config
from collective.fingerpointing.config import LOG_FORMAT
from collective.fingerpointing.config import PROJECTNAME

import logging
import time
import zc.lockfile

logger = logging.getLogger(PROJECTNAME)
logger.setLevel(logging.INFO)

logfile = fingerpointing_config.get('audit-log', None)
if logfile is None:
    logger.warn('No audit log file specified; audit log view will be disabled')
else:
    # if either of maxBytes or backupCount is zero, rollover never occurs
    maxBytes = int(fingerpointing_config.get('audit-log-max-size', 0))
    backupCount = int(fingerpointing_config.get('audit-log-old-files', 1))
    handler = logging.handlers.RotatingFileHandler(
        logfile, maxBytes=maxBytes, backupCount=backupCount)
    formatter = logging.Formatter(LOG_FORMAT)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.info('Logging audit information to ' + logfile)


def log_info(*args, **kwargs):
    """Log information to a file handling access from multiple instances.
    This code was taken from ZEO/ClientStorage.py.
    """
    # if no logfile was specified just log the event normally
    if logfile is None:
        logger.info(*args, **kwargs)
        return

    # otherwise, try to lock the logfile then writing to it
    lockfilename = logfile + '.lock'
    n = 0

    while 1:
        try:
            lock = zc.lockfile.LockFile(lockfilename)
            logger.info(*args, **kwargs)
            lock.close()
        except zc.lockfile.LockError:
            time.sleep(0.01)
            n += 1
            if n > 60000:
                raise
        else:
            break

    return



# -*- coding: utf-8 -*-
from collective.fingerpointing.logger import logfile as LOGFILE
from glob import glob
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class LogView(BrowserView):

    index = ViewPageTemplateFile('logview.pt')

    def __call__(self):
        return self.render()

    def render(self):
        return self.index()

    def available(self):
        return LOGFILE is not None

    def tail(self, f, n=100):
        """Return the last n lines of a file.
        See: http://stackoverflow.com/a/280083/644075
        """
        assert n >= 0
        pos, lines = n + 1, []
        while len(lines) <= n:
            try:
                f.seek(-pos, 2)
            except IOError:
                f.seek(0)
                break
            finally:
                lines = list(f)
            pos *= 2
        return lines[-n:]

    @property
    def _get_audit_log_files(self):
        """Return a list with the current audit log and all of its
        backups. The list is sorted with newer files first.
        """
        # retrieve the list of audit log backup files
        logfiles = sorted([
            f for f in glob(LOGFILE + '.*') if '.lock' not in f
        ], reverse=True)
        # include active audit log as first element
        logfiles.insert(0, LOGFILE)
        return logfiles

    @property
    def get_audit_log(self):
        """Return audit log with newer entries first."""
        lines = []
        # process all logs until we have enough lines
        for logfile in self._get_audit_log_files:
            with open(logfile, 'r') as fp:
                lines.extend(self.tail(fp))
            if len(lines) >= 100:
                break

        lines = sorted(lines[:100], reverse=True)
        return ''.join(lines)



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from plone.api.exc import InvalidParameterError
from Products.PluggableAuthService.interfaces.events import IGroupDeletedEvent
from Products.PluggableAuthService.interfaces.events import IPrincipalCreatedEvent
from Products.PluggableAuthService.interfaces.events import IPrincipalDeletedEvent
from Products.PluggableAuthService.interfaces.events import IUserLoggedInEvent
from Products.PluggableAuthService.interfaces.events import IUserLoggedOutEvent


def pas_logger(event):
    """Log authentication events like users logging in and loggin out."""
    # subscriber is registered even if package has not yet been installed
    # ignore any error caused by missing registry records
    try:
        record = IFingerPointingSettings.__identifier__ + '.audit_pas'
        audit_pas = api.portal.get_registry_record(record)
    except InvalidParameterError:
        return

    if audit_pas:
        user, ip = get_request_information()
        if IUserLoggedInEvent.providedBy(event):
            action = 'login'
            extras = ''
        elif IUserLoggedOutEvent.providedBy(event):
            action = 'logout'
            extras = ''
        elif IPrincipalCreatedEvent.providedBy(event):
            action = 'create'
            extras = u'principal={0}'.format(event.principal)
        elif IPrincipalDeletedEvent.providedBy(event):
            action = 'remove'
            extras = u'user={0}'.format(event.principal)
        if IGroupDeletedEvent.providedBy(event):
            action = 'remove'
            extras = u'group={0}'.format(event.principal)

        log_info(AUDIT_MESSAGE.format(user, ip, action, extras))
